package com.xynet.marketing.service.factory.coupon.merc.impl.base;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.xynet.marketing.entity.MemberCoupon;
import com.xynet.marketing.entity.MercCoupon;
import com.xynet.marketing.enums.CouponSendTypeEnum;
import com.xynet.marketing.enums.CouponValidTypeEnum;
import com.xynet.marketing.mapper.MercCouponMapper;
import com.xynet.marketing.service.MemberCouponService;
import com.xynet.marketing.service.MercProjectDeviceService;
import com.xynet.marketing.service.MercProjectGoodsService;
import com.xynet.marketing.service.factory.coupon.merc.MercCouponFactory;
import com.xynet.marketing.utils.Emptys;
import com.xynet.marketing.utils.collections.map.JHashMap;
import com.xynet.marketing.utils.collections.map.JMap;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * 优惠券工厂-基类
 *
 * @author hechunping
 * @date 2025/2/10
 */
@Service
@Slf4j
@AllArgsConstructor
public class MercCouponFactoryBase extends ServiceImpl<MercCouponMapper, MercCoupon> implements MercCouponFactory {

    private MercProjectGoodsService goodsService;

    private MercProjectDeviceService deviceService;

    private MemberCouponService memberCouponService;

    @Override
    public Integer saveOrUpdate(JMap<String, Object> param) {
        MercCoupon save = BeanUtil.copyProperties(param, MercCoupon.class);
        if (Emptys.check(save.getId())) {
            updateById(save);
            //保存商品
            if (!save.getIsAllGoods()) {
                //删除原来的商品
                goodsService.removeByActId(save.getId());
            }
            //保存设备
            if (!save.getIsAllDevice()) {
                //删除原来的设备
                deviceService.removeByActId(save.getId());
            }
        } else {
            save(save);
        }
        //保存商品
        if (!save.getIsAllGoods()) {
            //删除原来的商品
            goodsService.saveBatchByActId(save.getId(), "coupon", param.getListMap("goodsList"));
        }
        //保存设备
        if (!save.getIsAllDevice()) {
            deviceService.saveBatchByActId(save.getId(), "coupon", param.getListMap("deviceList"));
        }

        return save.getId();
    }

    @Override
    public void mercSend(JMap<String, Object> param) {
        List<Object> thirdMemberIdList = param.getList("thirdMemberIdList");
        if (!Emptys.check(thirdMemberIdList)) {
            throw new RuntimeException("会员列表不能为空！");
        }
        LambdaQueryWrapper<MercCoupon> lqw = new LambdaQueryWrapper<MercCoupon>()
                .eq(MercCoupon::getMercId, param.getInt("mercId"))
                .in(MercCoupon::getId, param.getList("couponIdList"));
        List<MercCoupon> couponList = list(lqw);
        if (!Emptys.check(couponList)) {
            throw new RuntimeException("没有可以赠送的优惠券！");
        }
        if (thirdMemberIdList.size() > 20) {
            throw new RuntimeException("一次最多赠送20个用户！");
        }
        couponList.forEach(coupon -> {
            thirdMemberIdList.forEach(memberId -> {
                memberCouponService.saveOrUpdate(modelSave(coupon, memberId.toString(), param.getString("remark"), null));
            });
        });
    }

    @Override
    public void registerSend(JMap<String, Object> param) {
        LocalDate today = LocalDate.now();
        String thirdMemberId = param.getString("thirdMemberId");
        //查找商家可赠送的优惠券
        LambdaQueryWrapper<MercCoupon> lqw = new LambdaQueryWrapper<MercCoupon>()
                .eq(MercCoupon::getMercId, param.getInt("mercId"))
                .eq(MercCoupon::getStatus, true)
                .eq(MercCoupon::getSendType, CouponSendTypeEnum.T_2.getCode())
                .and(wrapper -> wrapper.isNull(MercCoupon::getSendStartDate)
                        .or()
                        .le(MercCoupon::getSendStartDate, today))
                .and(wrapper -> wrapper.isNull(MercCoupon::getSendEndDate)
                        .or()
                        .ge(MercCoupon::getSendEndDate, today));
        List<MercCoupon> list = list(lqw);
        if (!Emptys.check(list)) {
            throw new RuntimeException("没有可赠送的优惠券！");
        }
        list.forEach(mercCoupon -> {
            //查询券是否已领取过了
            JMap<String, Object> existParam = new JHashMap<String, Object>()
                    .set("mercId", mercCoupon.getMercId())
                    .set("thirdMemberId", thirdMemberId)
                    .set("couponId", mercCoupon.getId());
            Map<String, Object> existNewMember = memberCouponService.obj(existParam);
            if (!Emptys.check(existNewMember)) {
                memberCouponService.saveOrUpdate(modelSave(mercCoupon, thirdMemberId, "新注册赠送", null));
            }
        });
    }

    @Override
    public void orderSend(JMap<String, Object> param) {
        String thirdMemberId = param.getString("thirdMemberId");
        LocalDate orderCreateTime = param.getTimestamp("orderCreateTime").toLocalDateTime().toLocalDate();
        //查找商家可赠送的优惠券
        LambdaQueryWrapper<MercCoupon> lqw = new LambdaQueryWrapper<MercCoupon>()
                .eq(MercCoupon::getMercId, param.getInt("mercId"))
                .eq(MercCoupon::getStatus, true)
                .eq(MercCoupon::getSendType, CouponSendTypeEnum.T_3.getCode())
                .and(wrapper -> wrapper.isNull(MercCoupon::getSendStartDate)
                        .or()
                        .le(MercCoupon::getSendStartDate, orderCreateTime))
                .and(wrapper -> wrapper.isNull(MercCoupon::getSendEndDate)
                        .or()
                        .ge(MercCoupon::getSendEndDate, orderCreateTime));
        List<MercCoupon> list = list(lqw);
        if (!Emptys.check(list)) {
            throw new RuntimeException("没有可赠送的优惠券！");
        }
        //TODO 判断设备条件
        list.forEach(mercCoupon -> {
            JSONObject sendConfig = JSONUtil.parseObj(mercCoupon.getSendConfig());
            Float limitAmount = sendConfig.getFloat("limitAmount");
            //判断支付金额是否 >= 优惠券最低金额
            if (param.getInt("payMoney") >= limitAmount * 100) {
                memberCouponService.saveOrUpdate(modelSave(mercCoupon, thirdMemberId, "购物后赠送", param.getString("orderId")));
            }
        });
    }

    @Override
    public void autoSend(JMap<String, Object> param) {
        LocalDate today = LocalDate.now();
        //获取自动赠送的列表
        List<String> enumByAutoSend = CouponSendTypeEnum.getEnumByAutoSend(true);
        //TODO 没完成


    }


    public JMap<String, Object> modelSave(MercCoupon mercCoupon, String memberId, String remark, String createOrderId) {
        LocalDate today = LocalDate.now();
        //有效时间
        boolean isDays = Objects.equals(mercCoupon.getValidType(), CouponValidTypeEnum.T_1.getCode());
        LocalDate validStartDate = isDays ? today : mercCoupon.getValidStartDate();
        LocalDate validEndDate = isDays ? today.plusDays(mercCoupon.getValidDay() - 1) : mercCoupon.getValidEndDate();
        MemberCoupon memberCoupon = new MemberCoupon()
                .setMercId(mercCoupon.getMercId())
                .setThirdMemberId(memberId)
                .setCouponId(mercCoupon.getId())
                .setCouponName(mercCoupon.getName())
                .setIsAllDevice(mercCoupon.getIsAllDevice())
                .setIsAllGoods(mercCoupon.getIsAllGoods())
                .setValidStartDate(validStartDate)
                .setValidEndDate(validEndDate)
                .setUseMinPrice(mercCoupon.getUseMinPrice())
                .setType(mercCoupon.getType())
                .setPrice(mercCoupon.getPrice())
                .setCreateDate(today)
                .setCreateOrderId(createOrderId)
                .setRemark(remark);
        return new JHashMap<>(BeanUtil.beanToMap(memberCoupon));
    }
}
