package com.xynet.marketing.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.xynet.marketing.entity.MercCoupon;
import com.xynet.marketing.mapper.MercCouponMapper;
import com.xynet.marketing.service.MercCouponService;
import com.xynet.marketing.service.MercProjectDeviceService;
import com.xynet.marketing.service.MercProjectGoodsService;
import com.xynet.marketing.service.factory.coupon.merc.MercCouponFactory;
import com.xynet.marketing.utils.Emptys;
import com.xynet.marketing.utils.FactoryUtils;
import com.xynet.marketing.utils.collections.map.JHashMap;
import com.xynet.marketing.utils.collections.map.JMap;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

/**
 * a
 * 商家优惠券服务-实现类
 *
 * @author hechunping
 * @date 2025/2/10
 */
@Service
@Slf4j
@AllArgsConstructor
public class MercCouponServiceImpl extends ServiceImpl<MercCouponMapper, MercCoupon> implements MercCouponService {

    private MercProjectGoodsService goodsService;
    private MercProjectDeviceService deviceService;

    private static String ACT_TYPE = "coupon";

    private LambdaQueryWrapper<MercCoupon> commonQueryWrapper(JMap<String, Object> param) {
        LambdaQueryWrapper<MercCoupon> lqw = new LambdaQueryWrapper<MercCoupon>()
                .eq(Emptys.check(param.getInt("id")), MercCoupon::getId, param.getInt("id"));
        return lqw;
    }

    @Override
    public Map<String, Object> obj(JMap<String, Object> param) {
        LambdaQueryWrapper<MercCoupon> lqw = commonQueryWrapper(param).last("limit 1");
        MercCoupon obj = getOne(lqw);
        Map<String, Object> vo = JHashMap.copy(obj);
        //反显设备信息
        if (!obj.getIsAllDevice()) {
            List<Map<String, Object>> deviceList = deviceService.listByActId(obj.getId(), ACT_TYPE);
            vo.put("deviceList", deviceList);
        }
        //反显商品信息
        if (!obj.getIsAllGoods()) {
            List<Map<String, Object>> goodsList = goodsService.listByActId(obj.getId(), ACT_TYPE);
            vo.put("goodsList", goodsList);
        }
        return vo;
    }

    @Override
    public Integer saveOrUpdate(JMap<String, Object> param) {
        MercCouponFactory mercCouponFactory = FactoryUtils.getService(MercCouponFactory.class, "merc_coupon_base");
        return mercCouponFactory.saveOrUpdate(param);
    }

    @Override
    public List<Map<String, Object>> list(JMap<String, Object> param) {
        LambdaQueryWrapper<MercCoupon> lqw = commonQueryWrapper(param).orderByAsc(MercCoupon::getId);
        List<MercCoupon> list = list(lqw);
        return JHashMap.copy(list);
    }

    @Override
    public Page<Map<String, Object>> page(JMap<String, Object> param) {
        LambdaQueryWrapper<MercCoupon> lqw = commonQueryWrapper(param).orderByAsc(MercCoupon::getId);
        Page<MercCoupon> page = new Page<>(param.getInt("pageNum"), param.getInt("pageSize"), true);
        Page<MercCoupon> list = page(page, lqw);
        return JHashMap.copy(list);
    }

    @Override
    public void updateSimple(JMap<String, Object> param) {

    }
}
